﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace WinForm_Backworker
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			this._backworker = new BackgroundWorker();
			// it's neccessary to set the properties.
			this._backworker.WorkerReportsProgress = true;
			this._backworker.WorkerSupportsCancellation = true;

			this._backworker.DoWork += new DoWorkEventHandler(backworker_DoWork);
			this._backworker.ProgressChanged += new ProgressChangedEventHandler(backworker_ProgressChanged);
			this._backworker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backworker_RunWorkerCompleted);
		}

		private BackgroundWorker _backworker = null;

		// body of thread
		void backworker_DoWork(object sender, DoWorkEventArgs e)
		{
			for (int i = 0; i < 100; i++)
			{
				Thread.Sleep(100);

				// trigger the event: ProgressChanged
				this._backworker.ReportProgress(i);

				// if receive the notification of Cancel.
				if (this._backworker.CancellationPending)
				{
					e.Cancel = true;
					return;
				}
			}
		}

		void backworker_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			// to do nothing.
		}

		void backworker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			// if thread is completed because of cancellation.
			if (e.Cancelled)
				this.labelSummary.Text = "Calculation is cancelled";

			this.btnCalculate.Enabled = true;
		}

		void backworker_OnCancel(object sender, CancelEventArgs e)
		{
			this._backworker.CancelAsync();
		}

		private void btnCalculate_Click(object sender, EventArgs e)
		{
			string temp = this.txtMax.Text.Trim();
			if (string.IsNullOrEmpty(temp))
				MessageBox.Show("Please input a valid integer.", "Validation", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			else
			{
				ProgressDlg progressDlg = new ProgressDlg();
				// if thread is executing, dialog will show the progress.
				this._backworker.ProgressChanged += new ProgressChangedEventHandler(progressDlg.OnProgressChanged);
				this._backworker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(progressDlg.OnCompleted);
				// if cancel in progressDlg, let me know and do something.
				progressDlg.OnCancel += new CancelEventHandler(backworker_OnCancel);

				this.btnCalculate.Enabled = false;
				progressDlg.Show();

				// start the thread.
				this._backworker.RunWorkerAsync();
			}
		}


	}
}
